/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * 
 */

package golfShop;

import java.io.*;
import java.math.BigDecimal;
import java.util.Enumeration;
import com.lutris.util.*;
import com.lutris.logging.Logger;
import com.lutris.logging.LogChannel;
import com.lutris.appserver.server.session.*;
import com.lutris.appserver.server.user.*;
import com.lutris.appserver.server.*;
import com.lutris.appserver.server.httpPresentation.*;

import golfShop.data.user.UserDO;
import golfShop.business.user.*;
import golfShop.business.item.*;
import golfShop.business.cart.Cart;

/**
 * Application class for this specific application.  This implements
 * standard startup and shutdown methods and well as application specific
 * ones.
 */
public class GolfShop extends StandardApplication {
    /*
     * This is the global Config object that is shared by the 
     * whole application. If there was an error reading the file, this
     * will be null.
     *
     * @see Config
     */
    public static Config config = null;

    /**
     * User manager for this application.
     */
    private GolfShopUserManager userManager = null;

    /**
     * Return the UserManager for this server.
     */
    public GolfShopUserManager getUserManager() {
        return userManager;
    }
    
    
    /**
     * Start the application.
     * 
     * @param appConfig Configuration object.
     * @exception HarmonyAppException If an error occurs starting the
     *  application.
     */
    public void startup(Config appConfig) throws ApplicationException {
        /*
         * This will create the config and sessionManager objects.
         */
        super.startup(appConfig);
        if (logChannel != null) {
            logChannel.write(Logger.INFO, 
                             "Welcome to the Lutris GolfShop application!");
        }
        config = appConfig;
        try {
            userManager = new GolfShopUserManager(this);
            UserDO.InitializeStorageOption();
        } catch (Exception e) {
            // N.B. If shutdown methods are available for session and user
            // manager, call them.
            userManager = null;

            // This application isn't designed to be restarted.
            state = DEAD;
            if (e instanceof ApplicationException) {
                throw (ApplicationException)e;
            } else {
                throw new ApplicationException("failure to initialize GolfShop", e);
            }
        }
    }

    /**
     * Shutdown the application.
     */
    public void shutdown() {
        userManager = null;
        super.shutdown();
    }


    public boolean requestPreprocessor(HttpPresentationComms comms)
        throws Exception {
        
        if (logChannel != null) {
            logChannel.write(Logger.DEBUG,
                             "URI=\"" + comms.request.getRequestURI() + "\"");
        }
        
        super.requestPreprocessor(comms);
        if (comms.session == null) {
            // Session not setup; so its not a presentation object.
            return false;
        }

        String pp = comms.request.getApplicationPath();
        if (!pp.endsWith("/"))
            pp += "/";
        String loginDir = pp + "login/";
        String mediaDir = pp + "media/";
        String page = comms.request.getRequestURI();
        if (page.startsWith(loginDir) || page.startsWith(mediaDir)) {
            /*
             *   Let them go to the login and logout areas whether or not
             *   they are logged in.
             */
            return false;
        } else {
            /*
             *   All other pages require the user to be logged in.
             */
            if (comms.session.getUser() != null)
                // It's ok, they are logged in already.
                return false;
            else {
                // They are not logged in! Redirect to login page.
                String url = pp + "login/Login.po";
                throw new ClientPageRedirectException(url);
            }
        }
    }

    /*
     * Generate an HTML table of sessions.
     */
    private String makeSessionTable(Enumeration allSessions) {
        StringBuffer results = new StringBuffer();

        results.append("<TABLE><TR><TD ALIGN=CENTER><B>Username</B></TD>" +
                       "<TD ALIGN=CENTER><B>Current total</B></TD></TR>\n");
        while (allSessions.hasMoreElements()) {
            String sessionKey = (String)allSessions.nextElement();
            Session session;
            try {
                session = sessionManager.getSession(sessionKey);
            } catch (SessionException except) {
                // Skip session that may have just become invalid
                continue;
            }
            Cart cart = null;
            String username = null;
            String totalStr = null;
            
            User user = session.getUser();
            if (user != null) {
                username = user.getName();
            } else {
                username = "<I>Not logged in yet</I>";
            }

            SessionData sd = session.getSessionData();
            try {
                cart = (Cart)sd.get("cart");
            } catch (KeywordValueException kve) {
            }
            if (cart != null) {
                double total = cart.getTotal();
                BigDecimal total2 = new BigDecimal(total);
                total2 = total2.setScale(2, BigDecimal.ROUND_HALF_UP);
                totalStr = "$" + total2;
            } else {
                totalStr = "<I>No cart</I>";
            }
            results.append("<TR><TD ALIGN=CENTER>");
            results.append(username);
            results.append("</TD><TD ALIGN=RIGHT>");
            results.append("</TD></TR>\n");
        }
        results.append("</TABLE>\n");
        return results.toString();
    }

    /*
     * The MultiServer's admin app will notice that this function exists
     * and will call it when showing the state of this application.
     * This is my chance to display info specific to this application.
     */
    public String toHtml() {
        Enumeration allSessions;
        try {
            allSessions = sessionManager.getSessionKeys();
        } catch (SessionException except) {
            // Return error as HTML.
            return except.getClass().getName() + " " + except.getMessage();
        }
        if (!allSessions.hasMoreElements()) {
            return "There are no current sessions.";
        } else {
            return makeSessionTable(allSessions);
        }
    }
}
